#[derive(Debug, StructOpt)]
#[structopt(
name = "music_bg",
about = "Listens to your mpris interface and sets you a cool background"
)]
pub struct Opt {
    #[structopt(subcommand)]
    pub cmd: Option<RunMode>,
}

#[derive(StructOpt, Debug)]
pub enum RunMode {
    #[structopt(name = "run", about = "Run d-bus listener")]
    Run,
    #[structopt(
    name = "config",
    about = "Create or replace default configuration file"
    )]
    GenConf,
    #[structopt(
    name = "test",
    about = "Process image file according to your ~/.mbg.toml"
    )]
    TestPic(TestingPic),
    #[structopt(
    name = "completions",
    about = "Generate completion for your shell"
    )]
    GenComp(ShellType),
}

#[derive(StructOpt, Debug)]
pub struct TestingPic {
    #[structopt(
    name = "input",
    about = "Image file to process"
    )]
    pub input: String,
    #[structopt(
    long,
    about = "Processed image output file",
    name = "out",
    default_value = "/dev/shm/test.png"
    )]
    pub output: String,
    #[structopt(
    short,
    about = "Set processed image as wallpaper",
    name = "wall"
    )]
    pub wallpaper: bool,
}

#[derive(StructOpt, Debug)]
pub enum ShellType {
    #[structopt(
    about = "Generates a .bash completion file for the Bourne Again SHell (BASH)",
    name = "bash"
    )]
    Bash,
    #[structopt(
    about = "Generates a .fish completion file for the Friendly Interactive SHell (fish)",
    name = "fish"
    )]
    Fish,
    #[structopt(
    about = "Generates a completion file for the Z SHell (ZSH)",
    name = "zsh"
    )]
    Zsh,
    #[structopt(
    about = "Generates a completion file for PowerShell",
    name = "ps"
    )]
    PowerShell,
    #[structopt(
    about = "Generates a completion file for Elvish",
    name = "elvish"
    )]
    Elvish,
}

impl From<ShellType> for Shell {
    fn from(shell: ShellType) -> Self {
        match shell {
            ShellType::Bash => { Shell::Bash }
            ShellType::Fish => { Shell::Fish }
            ShellType::Zsh => { Shell::Zsh }
            ShellType::PowerShell => { Shell::PowerShell }
            ShellType::Elvish => { Shell::Elvish }
        }
    }
}

pub fn generate_completions(shell: ShellType) -> MBGResult<()> {
    Opt::clap().gen_completions(env!("CARGO_PKG_NAME"), shell.into(), ".");
    info!("Generated completion file in _music_bg");
    Ok(())
}